import operator
import math
import struct

# Y is clockwise to right of X

class vector:
    def __init__ (self, v):
        self.v=v
        self.dim = len(v)
    
    
        
    def __iadd__ (self,v):
        assert self.dim == v.dim
        self.v = [n[0]+n[1] for n in zip(self.v,v.v)]
        return self
    def __isub__ (self,v):
        assert self.dim == v.dim
        self.v = [n[0]-n[1] for n in zip(self.v,v.v)]
        return self
    def __imul__ (self,s):
        self.v = [n*s for n in self.v]
        return self
        
        
        
    def __add__ (self,v):
        assert self.dim == v.dim
        return vector ([n[0]+n[1] for n in zip(self.v,v.v)])
    def __sub__ (self,v):
        assert self.dim == v.dim
        return vector ([n[0]-n[1] for n in zip(self.v,v.v)])
    def __mul__ (self,s):
        return vector ([n*s for n in self.v])
        
        
        
        
    def clone (self):
        return vector(self.v)
        
    def len(self):
        return math.sqrt (reduce (operator.add,[n**2 for n in self.v]))
    
    def normalize(self):
        l = self.len()
        if (l == 0):
            self.v[0]=1
            l = self.len() # dodge the bullet
           
        return self * (1/l)
    
    def dot(self,v):
        return reduce (operator.add, [n[0]*n[1] for n in zip (self.v,v.v)])
    def rotate_90_ccw(self):
        assert self.dim == 2
        return vector ([self.v[1],-self.v[0]])
     
    def integer(self):
        return vector([int(x) for x in self.v])
    def str (self):
        return self.v
    def repr (self):
        return self.v
    def write (self,f):
        f.write(struct.pack(">Bi",1,len(self.v))) # 1 is format version
        for x in self.v:
            f.write(struct.pack(">d",x))
    
  
def vector_from_file (f):
    version = struct.unpack ("B",f.read(1))[0]
    assert version == 1
    dim = struct.unpack (">i",f.read(4))[0]
    v = []
    for i in range(dim):
        v.append (struct.unpack (">d",f.read(8))[0])
    return vector (v)
    
def vector3(v1,v2,v3):
    return vector([v1,v2,v3])
    
def vector2(v1,v2):
    return vector([v1,v2],2)
    
    