import pygame
from math_util import vector
from colors import * 
from pygame.locals import *
from math import sin
import struct

man_offset = vector([-32,-25])
man_jump_vector = vector([0,-5])
flag_offset = vector([-80,-80])

progress_file_name = "progress"
class Navigator:
    def __init__ (self,engine):
        self.engine = engine
        self.map_img = pygame.image.load("map.png")
        self.man_img = pygame.image.load("man.png")
        self.flag_img = pygame.image.load("flag.png")
        self.start_msg_img = pygame.image.load("start_message.png")
        self.man_img.set_colorkey ([0,0,0])
        self.flag_img.set_colorkey ([0,0,0])
        self.start_msg_img.set_colorkey ([0,0,0])
        
        self.done_up_to = 0
        try:
            f = open(progress_file_name,"rb")
            self.done_up_to = struct.unpack ("B",f.read(1))[0]
            f.close()
        except IOError:
            pass
        #print "done upto",self.done_up_to
        self.max_pos = 5
        self.min_pos = 0

        self.max_max_pos = 5
        
        self.from_pos = min (self.max_max_pos,self.done_up_to)
        self.pos = min (self.max_max_pos,self.done_up_to)
        
        self.max_pos = self.done_up_to
        self.man_cycle = 0
        self.man_travelled = 0.0
        self.travelling = False
        self.pos_vectors =[vector(a) for a in [[86,417],[230,417],[383,409],[470,287],[296,236],[346,103]]]
        self.forward_keys = [K_RIGHT,K_RIGHT,K_UP,K_LEFT,K_UP,K_UP]
        self.backward_keys = [K_UP,K_LEFT,K_LEFT,K_DOWN,K_RIGHT,K_DOWN]
        
        self.level_selected = False
        self.selected_level_index = 0
        self.msg_cycle = 0
        self.default_man_speed = 1.0
        self.man_speed = self.default_man_speed
        
    def render (self):
        pygame.draw.rect(self.engine.surface, red, [0,0,50,50])
        self.engine.surface.blit(self.map_img,[0,0])
        mpos = (self.pos_vectors[self.from_pos]*(1.0-self.man_travelled)+self.pos_vectors[self.pos]*self.man_travelled)+\
              man_offset+man_jump_vector*sin(self.man_cycle)**2
            
        self.engine.surface.blit(self.man_img,mpos.v)
        
        for i in range (self.done_up_to):
            self.engine.surface.blit(self.flag_img,(self.pos_vectors[i]+flag_offset).v)
            
        if (self.done_up_to == 0 and self.msg_cycle < 0.8):
            self.engine.surface.blit(self.start_msg_img,[30,280])
    def level_complete(self):
        
        self.done_up_to = max (self.selected_level_index + 1,self.done_up_to)
        f = open (progress_file_name,"wb")
        f.write(struct.pack ("B",self.done_up_to))
        f.close()
        self.max_pos = min (self.max_max_pos,self.done_up_to)
        
        self.from_pos = self.pos
        self.pos = min (self.pos+1,self.max_max_pos)
        self.level_selected = False
        self.man_speed = 0.5
        self.man_travelled = 0.0
    def tick (self,t):
        self.man_cycle += t*2.0
        self.man_travelled += t*self.man_speed
        self.msg_cycle += t
        if self.msg_cycle > 1.6:
            self.msg_cycle = 0
        if (self.man_travelled >= 1.0):
            self.man_travelled = 0
            self.from_pos = self.pos
            self.travelling = False
        
        if (not self.travelling): 
            new_pos = self.pos        
            if (self.engine.is_key_pressed(self.backward_keys[self.pos])):
                new_pos -= 1
            elif (self.engine.is_key_pressed(self.forward_keys[self.pos])):
                new_pos += 1
            new_pos = max (min (new_pos,self.max_pos),self.min_pos)
            if (new_pos != self.pos):
                self.pos = new_pos
                self.travelling = True
                self.man_travelled = 0
                self.man_speed = self.default_man_speed
        if (self.engine.is_key_pressed(K_RETURN) or self.engine.is_key_pressed(K_SPACE)):
            self.level_selected = True
            self.selected_level_index = self.pos