from math_util import vector2, vector, vector_from_file
import struct
from colors import *

POLY_TYPE_NORMAL = 0
POLY_TYPE_NONSTICK = 1

poly_colours = [red,green]

class Polygon:
    def __init__ (self, v,type = POLY_TYPE_NORMAL):
        self.points = []
        self.type = type
        self.message = None
        
        if (isinstance (v[0],vector)):
            self.points = v[:]
        else:
            self.points = [vector(a) for a in v]
            
    def get_points (self):
        for p in self.points:
            yield p
            
    def get_lines (self):
        last_p = None
        for p in self.points:
            if (last_p):
                yield (last_p,p,self)
            last_p = p
        yield (last_p , self.points[0],self)
    
    def write (self,f):
        if self.message:
            
            s = struct.pack(">BBBB%usi"%len(self.message),3,self.type,self.message_index,len(self.message),self.message,len(self.points))
            
            f.write(s) # 3 is format version
        else:
            f.write(struct.pack(">BBBi",3,self.type,0,len(self.points))) # 3 is format version
        
        for p in self.points:
            p.write(f)
    def load_message (self,index,message):
        self.message_index = index
        self.message = message
        #print "Message set %u:%s"%(self.message_index,self.message)

def polygon_from_file(f):
    version = struct.unpack ("B",f.read(1))[0]
    
    if (version >= 2):
        t = struct.unpack (">B",f.read(1))[0]
    else:
        t = POLY_TYPE_NORMAL
    message_index = 0
    if (version >= 3):
        message_index = struct.unpack (">B",f.read(1))[0]
    if (message_index):
        l = struct.unpack (">B",f.read(1))[0]
        message_string = f.read(l)
        
        
    dim = struct.unpack (">i",f.read(4))[0]
    v = []
    for i in range(dim):
        v.append (vector_from_file(f))
    new_poly = Polygon (v,type=t)
    if message_index:
        new_poly.load_message (message_index,message_string)
    return new_poly

cell_size = [200,200]
    
def closer_to_horizontal (l):
    return abs(l[1].v[1]-l[0].v[1]) < abs(l[1].v[0]-l[0].v[0])
'''
class Poly_line:
        def __init__(self,v1,v2,poly):
            self.v1=v1
            self.v2=v2
            self.poly=poly'''
    
class Cell:
    def __init__(self):
        self.line_set = set([])
    def add_line (self,l,delete=False):
        
        if (delete):
            #print "delete line"
            self.line_set -= set([l])
        else:
            #print "add line",self
            self.line_set |= set([l])
    
class Polygon_world:
    def __init__ (self,size):
        self.size = size
        self.polys = []
        self.num_cells = [size[0]/cell_size[0]+1,size[1]/cell_size[1]+1]
        self.cells = [[Cell() for n in range(self.num_cells[1])] for n in range(self.num_cells[0])]
    def add_poly(self,poly):
        self.polys.append (poly)
        self.refresh_cell_lines_for_poly(poly)

    def refresh_cell_lines_for_poly (self,poly, delete = False):
        for line_orig in poly.get_lines():
            x_index = 0
            y_index = 1
            if not closer_to_horizontal (line_orig):
                x_index = 1
                y_index = 0
                
            l=[0,0]
            if (line_orig[0].v[x_index] < line_orig[1].v[x_index]): #ensure we're going from left to right
                l[0] = line_orig[0].clone()
                l[1] = line_orig[1].clone()
            else:
                l[0] = line_orig[1].clone()
                l[1] = line_orig[0].clone()
            
            line = l[1]-l[0]
            len_len = line.len()
            horiz_len = (l[1].v[x_index]-l[0].v[x_index])
            if (horiz_len == 0):
                self.get_cell (l[0]).add_line(line_orig)
                continue
            step_s = float(cell_size[x_index])
            step_v = line*(step_s/horiz_len)
            pos_s = 0 # position along x axis
            pos_v = l[0].clone()
            done = False
            self.get_cell (pos_v).add_line(line_orig,delete)
            to_next_wall = cell_size[x_index] - (l[0].v[x_index]%cell_size[x_index])
            pos_s += to_next_wall
            pos_v += step_v*(to_next_wall/step_s)
            just_to_the_side = vector([0,0])
            just_to_the_side.v[x_index]=1
            #print "horiz_len=",horiz_len,"step_s=",step_s,"step_v=",step_v.v
            while(not done):
                if (pos_s > horiz_len):
                    pos_v = l[1]
                    self.get_cell (pos_v).add_line(line_orig,delete)
                    break
                
                self.get_cell (pos_v+just_to_the_side).add_line(line_orig,delete)
                self.get_cell (pos_v-just_to_the_side).add_line(line_orig,delete)
                
                pos_v += step_v
                pos_s += step_s 
                #print "horiz_len=",horiz_len,"step_s=",step_s                
            
    def get_cell (self,p):
        #print "get cell:",p.v
        return self.cells[int(p.v[0]/cell_size[0])][int(p.v[1]/cell_size[1])]
    
    def get_polys (self):
        for p in self.polys:
            yield p
            
    def line_getter (self, p,x_offset,y_offset, hard_only = True, return_poly = False):
        first_x = int((p-x_offset).v[0]/cell_size[0])
        last_x = int((p+x_offset).v[0]/cell_size[0])
        first_y  = int((p-y_offset).v[1]/cell_size[1])
        last_y  = int((p+y_offset).v[1]/cell_size[1])
        line_set = set([])
        
        first_x = max (0,first_x)
        last_x = min (self.num_cells[0]-1,last_x)
        first_y = max (0,first_y)
        last_y = min (self.num_cells[1]-1,last_y)
        
        for x in range (first_x,last_x+1):
            for y in range (first_y,last_y+1):
                line_set |= self.cells[x][y].line_set
        
        for l in line_set:
            yield (l)
        
        
        #for p in self.polys:
        #    for l in p.get_lines():
        #        yield l
    
    
    def write (self,f):
        f.write(struct.pack(">Bi",1,len(self.polys))) # 1 is format version
        for p in self.polys:
            p.write(f)
    def read (self,f):
        version = struct.unpack ("B",f.read(1))[0]
        assert version == 1
        dim = struct.unpack (">i",f.read(4))[0]
        #print dim
        
        
        for i in range(dim):
            self.add_poly (polygon_from_file(f))
        #print f.tell( ) 

            
    def get_nearest_poly (self,pos,r):
        for p in self.polys:
            for l in p.get_lines():
                start_to_pos = pos - l[0]
                length = (l[1]-l[0]).len()
                line_dir = (l[1]-l[0]).normalize()
                pos_on_line = start_to_pos.dot(line_dir)
                line_normal = line_dir.rotate_90_ccw()
                distance_from_line = start_to_pos.dot (line_normal)
                abs_dist_from_line = abs(distance_from_line)
                if (abs_dist_from_line >= r):
                    continue
                if (pos_on_line >= 0 and pos_on_line <= length):
                    return p
                else:
                    pos_from_end = (pos-l[0])
                    end_distance = pos_from_end.len()
                    if (end_distance < r):
                        return p
                    pos_from_end = (pos-l[1])
                    end_distance = pos_from_end.len()
                    if (end_distance < r):
                        return p        
        return None
                
class Polygon_editor:
    def __init__ (self,w):
        self.world = w
        self.new_poly = None
        self.next_poly_type = POLY_TYPE_NORMAL
        self.message = None
    def add_point (self,p,final = False):
        poly = self.new_poly
        
        if not self.new_poly:
            self.new_poly = Polygon([p],self.next_poly_type)
            self.world.polys.append(self.new_poly)
        else:
            self.world.refresh_cell_lines_for_poly(self.new_poly, delete = True)
            self.new_poly.points.append(p)
            if final:
                self.new_poly = None
        if (poly):        
            self.world.refresh_cell_lines_for_poly(poly)
        
    def r_mouse_down (self,p):
        if (self.new_poly):
            self.add_point (p,final = True)
        else:
            poly = self.world.get_nearest_poly (p,10)
            if (poly):
                if self.message:
                    #load poly with message
                    poly.load_message (self.message_index,self.message)
                else:#delete poly
                    self.world.refresh_cell_lines_for_poly(poly,delete = True)
                    self.world.polys.remove(poly)
            else:
                if self.next_poly_type == POLY_TYPE_NONSTICK:
                    self.next_poly_type = POLY_TYPE_NORMAL
                    print "POLY_TYPE_NORMAL"
                else:
                    self.next_poly_type = POLY_TYPE_NONSTICK
                    print "POLY_TYPE_NONSTICK"
    def set_poly_message (self,index,message):
        self.message_index = index
        self.message = message
        
    def mouse_down (self,p):
        self.add_point (p,final = False)        
    