import pygame
from math_util import vector
from pygame.locals import *
from functools import partial

eng = None

class Engine:
    '''A class inheriting from engine must implement tick and render
    and call run to start'''
    
    
    def __init__ (self, width, height, name):
        global eng
        self.time_slow_factor = 1.0
        self.paused = False
        self.key_state = {}
        self.game_key_state = {}
        self.key_handlers = {}
        self.paused = False
        self.pausable = True
        self.step = 1.0/100
        self.width = width
        self.height = height
        self.screen_size = vector([width,height])
        self.blob = None
        self.focus = None
        eng = self # for console
        
        pygame.init()
        video_flags = DOUBLEBUF|  HWSURFACE
        self.surface = pygame.display.set_mode((width, height), video_flags)
        pygame.display.set_caption (name)
    
        
    def add_key_handler (self, k, handler):
        self.key_state[k] = False
        self.game_key_state[k] = False
        self.key_handlers[k] = handler
        
    def is_key_pressed (self,k):
        return self.game_key_state[k]
        
    def to_screen (self,v):
        if isinstance (v,vector):
            return v - self.screen_pos
        elif isinstance (v,list) and len(v) == 4:
            return [v[0]-self.screen_pos.v[0],v[1]-self.screen_pos.v[1],v[2],v[3]]
    def to_world (self,v):
        
        return v + self.screen_pos
      
        
    def run (self):
        done = False
        last_time = pygame.time.get_ticks()
        target_sim_time = 0.0
        sim_time = 0.0
        frames = 0
        last_fps_output = 0
        fps_period = 5000
        
        while not done:
            time = pygame.time.get_ticks()
            elapsed = ((time - last_time)/1000.0)/self.time_slow_factor # seconds
            last_time = time
            elapsed = min (elapsed,1.0/20)
            if (not self.paused):
                target_sim_time += elapsed
        
            event = pygame.event.poll()
            
            if event.type == QUIT:
                done = True 
                
            if (event.type == KEYDOWN):
                if (self.key_state.has_key(event.key)):
                    self.key_state[event.key]=True
            if (event.type == KEYUP):
                if (self.key_state.has_key(event.key)):
                    self.key_state[event.key]=False
                    
            if (not self.paused):
                for k in self.key_state:
                    if self.key_state[k] != self.game_key_state[k]:
                        s = self.game_key_state[k] = self.key_state[k]
                        if self.key_handlers.has_key(k):
                            if (self.key_handlers[k]):
                                self.key_handlers[k](s)
                        
            if (event.type == KEYDOWN):
                if (event.unicode == 'p' and self.pausable):
                    self.paused = not self.paused
            if (event.type == MOUSEBUTTONDOWN  and event.button == 1): # < 4 is not a silly wheel event
                if (self.mouse_down):
                    self.mouse_down(vector(event.pos))
            if (event.type == MOUSEBUTTONDOWN  and event.button == 3): # < 4 is not a silly wheel event
                if (self.r_mouse_down):
                    self.r_mouse_down(vector(event.pos))
                    
              
            this_time = 0
            
            while (target_sim_time > sim_time ):#and this_time < 5):
                this_time += 1
                self.tick(self.step)
                sim_time += self.step
            
            
            if (self.focus):
                self.screen_pos = (self.focus.pos - self.screen_size*0.5).integer()
                
            self.render()
            pygame.display.flip()
            
            
            frames += 1
            if ((time - fps_period ) > last_fps_output ):
                last_fps_output = time
                
                #print "FPS = " , frames*1000/fps_period
                frames = 0

    def point_to_world_forwarder (self, func):
        '''returns a function that can be passed a vector and will
        call func with the vector, converted to world coordinates'''
        return partial (self.call_func_with_world_point,func)
        
    def call_func_with_world_point (self,func,v):
        return func (self.to_world(v))
        
    def console (self):
        print "CONSOLE"
        line = ""
        print ">"
        while (line != "c"):
            print line
            try:
                if (len(line)):
                    exec (line)
            except SyntaxError, s:
                print s
            '''except RuntimeError, s:
                print s
            except TypeError, s:
                print s
            except  NameError, s:
                print s
            except  AttributeError, s:
                print s
                '''
            print ">" ;
            line = raw_input()
            
    def add_object (self,o):
        self.level.add_object (o)    
    def remove_object (self,o):
        self.level.remove_object (o)