from math_util import vector


class Stringlet:
    def __init__(self,string,pos):
        self.string = string
        self.mass = 4.0
        self.inv_mass = 1.0/self.mass
        self.vel= vector ([0,0])
        self.pos=pos
        self.young=150000.0
        self.nat_len = 10.0 #between this and the next
        self.next = 0
        self.fixed = False
        self.debug = False
        self.tension_force = vector([0,0])
    
    def set_natural_length (self):
        if (self.next):
            
            self.nat_len = (self.next.pos - self.pos).len()
            
    def change_vel (self,t):
       
        if not self.fixed:
            self.vel += self.string.gravity * t
        if (self.next):
            self.v = (self.next.pos - self.pos)
            self.vn = self.v.normalize()
            self.tension = self.young * (self.v.len() - self.nat_len)/self.nat_len
            if (self.tension > 0):
                self.tension_force = self.vn * self.tension
                f = self.tension_force *t
                self.vel +=  f * self.inv_mass
                self.next.vel -= f * self.next.inv_mass  #note other mass
            else:
                self.tension_force = vector ([0,0])
        if (self.fixed):
            self.vel = vector([0,0])
    
    def dampen (self,t):
        if (self.next):
            rel_vel = self.next.vel - self.vel
            
            inline = self.vn.dot (rel_vel)
            l = rel_vel.len()
            
            rel_vel *= 0.0013 ** t
            self.next.vel = self.vel + rel_vel           
            self.vel = self.next.vel - rel_vel    
    def change_pos (self,t):
        self.pos += self.vel * t
        
    def set_mass (self,m):
        self.mass = m
        self.inv_max = 1.0/m
    def get_tension(self):
        return self.tension_force.clone()
    
default_gravity = vector([0,0])

class String:
    
    def __init__ (self, start_pos,step,l):
        pos =  start_pos
        self.gravity = default_gravity
        self.stringlets = []
        self.def_len = 10.0
        self.end_section_len = 10.0
        last = 0
        for i in range (l):
            new = Stringlet(self,pos.clone())
            self.stringlets.append (new)
            if (last):
                last.next=new
            pos += step
            last = new
        self.stringlets[0].fixed = True
        
        #self.stringlets[l-1].inv_mass = 0.06
        
        #self.set_natural_lengths()
        
    def set_natural_lengths(self):
        for s in self.stringlets:
            s.set_natural_length()
        
    def tick (self,t):
        for s in self.stringlets:
            s.change_vel(t)
        for s in self.stringlets:
            s.dampen(t)
            s.change_pos(t)
    
    def get_end (self,end):
        if (end == 0):
            return self.stringlets[0]
        else:
            return self.stringlets[len(self.stringlets)-1]
    def set_end_pos (self,end,pos):
        self.get_end(end).pos = pos
    def set_end_vel (self,end,vel):
        self.get_end(end).vel = vel
    def get_end_pos (self,end):
        return self.get_end(end).pos.clone()
    def get_end_vel (self,end):
        return self.get_end(end).vel.clone()
    def extend_end (self,end):
        assert end == 0
        old_end = self.get_end(end)
        new_end = Stringlet(self,old_end.pos.clone())
        self.stringlets.insert(0,new_end)
        old_end.fixed = False
        new_end.fixed = True
        new_end.next = old_end
    def pop (self,end):
        assert end == 0
        self.stringlets.pop(0)
        self.get_end(end).fixed = True
    def lengthen (self,splitable):
        if (self.end_section_len >= self.def_len):
            if (splitable):
                self.end_section_len = self.def_len/2.0
                self.extend_end(0)
                self.stringlets[1].pos = (self.stringlets[0].pos + self.stringlets[2].pos)*0.5
                self.stringlets[2].nat_len = self.def_len
                self.end_section_len += 0.25
        else:
            self.end_section_len += 0.25
        self.stringlets[0].nat_len = self.end_section_len
        self.stringlets[1].nat_len = self.end_section_len
        
    def shorten (self):
        
        if (self.end_section_len <= self.def_len/2.0):
            self.end_section_len = self.def_len
            old_pos = self.stringlets[0].pos.clone()
            self.pop(0)
            self.stringlets[0].pos = old_pos
        self.end_section_len -= 0.25
        self.stringlets[0].nat_len = self.end_section_len
        self.stringlets[1].nat_len = self.end_section_len
        
        
        
    def get_len (self):
        return len(self.stringlets)
    def positions (self):
        '''enumarates positions of stringlets'''
        for s in self.stringlets:
            yield(s.pos)
    def debug (self):
        
        for s in self.stringlets:
            print s.nat_len
            