from math_util import vector
from functools import partial
from pygame.locals import *


class World_object:
    def __init__ (self, pos):
        self.pos = pos
        self.vel = vector([0,0])
        self.tick_fns = [self.wo_tick_fn ]
        self.radius = 10
    def wo_tick_fn (self,t):
        self.pos += self.vel*t
    def tick (self,t):
        for fn in self.tick_fns:
            fn (t)
    def make_heavy(self):
        self.tick_fns.append(partial (gravity_func,self))
    def bounding_box (self):
        pos = self.pos
        radius = self.radius
        return [pos.v[0]-radius,pos.v[1]-radius,radius*2,radius*2]
    def set_mass (self,m):
        self.mass = m
        self.inv_mass = 1.0/m
    
class Circle_collider_object (World_object):
    def __init__(self,pos):
        World_object.__init__ (self,pos)
        self.tick_fns.append (self.cco_tick_fn)
        self.radius = 30
        self.on_ground = False
        self.to_x_edge = vector([self.radius+5,0])
        self.to_y_edge = vector([0,self.radius+5])
        self.collided_line = None
    def cco_tick_fn (self,t):
        pass
        
    def tick (self,t):
        World_object.tick(self,t)
        self.on_ground = False
        self.collided_line = None
        self.do_line_collisions (t)

    def do_line_collisions (self,t):
        for l in self.line_getter(self.pos,self.to_x_edge,self.to_y_edge):
            start_to_pos = self.pos - l[0]
            length = (l[1]-l[0]).len()
            line_dir = (l[1]-l[0]).normalize()
            pos_on_line = start_to_pos.dot(line_dir)
            line_normal = line_dir.rotate_90_ccw()
            distance_from_line = start_to_pos.dot (line_normal)
            
            correction = None
            if (distance_from_line >= self.radius or distance_from_line <= 0):
                continue
            if (pos_on_line >= 0 and pos_on_line <= length):
                correction = line_normal * (self.radius - distance_from_line)
            else:
                pos_from_end = (self.pos-l[0])
                end_distance = pos_from_end.len() 
                if (end_distance < self.radius and end_distance != 0):
                    correction = pos_from_end* ((self.radius-end_distance) / end_distance)
                else:
                    pos_from_end = (self.pos-l[1])
                    end_distance = pos_from_end.len() 
                    if (end_distance < self.radius and end_distance != 0):
                        correction = pos_from_end* ((self.radius-end_distance) / end_distance)
            if (correction):
                self.collided_line = l
                self.pos += correction
                correction_unit = correction.normalize()
                vel_towards_surface = -self.vel.dot(correction_unit)
                if vel_towards_surface > 0:
                    self.vel += correction_unit * vel_towards_surface
                if (-correction.v[1] > abs(correction.v[0]*0.3)):
                    self.on_ground = True
    
    def get_nearest_land (self,pos,r, line_test = None):
        x_vec = vector([r,0])
        y_vec = vector([0,r])
        
        for l in self.line_getter(pos,x_vec,y_vec):
            if line_test and not line_test(l):
                continue
            start_to_pos = pos - l[0]
            length = (l[1]-l[0]).len()
            line_dir = (l[1]-l[0]).normalize()
            pos_on_line = start_to_pos.dot(line_dir)
            line_normal = line_dir.rotate_90_ccw()
            distance_from_line = start_to_pos.dot (line_normal)
            abs_dist_from_line = abs(distance_from_line)
            if (abs_dist_from_line >= r):
                continue
            if (pos_on_line >= 0 and pos_on_line <= length):
                return l[0] + line_dir * pos_on_line
            else:
                pos_from_end = (pos-l[0])
                end_distance = pos_from_end.len()
                if (end_distance < r):
                    return l[0].clone()
                pos_from_end = (pos-l[1])
                end_distance = pos_from_end.len()
                if (end_distance < r):
                    return l[1].clone()
        return None
    
    def set_line_getter (self, line_getter):
        '''Pass a function which will take 2d point and yield a set of lines which
        may lie near it. Each yield value should be a list of two vectors. The second
        vector would be the right hand end of the line if the collidable side of the line
        is face up'''
        self.line_getter = line_getter
        
        
class Object_mover:
    def __init__ (self, object, engine):
        self.object = object
        self.engine = engine
        
    def tick (self,t):
        power = 2000
        if (self.engine.is_key_pressed(K_UP)):
            self.object.vel += vector ([0,-power]) *t
        if (self.engine.is_key_pressed(K_DOWN)):
            self.object.vel += vector ([0,power])*t
        if (self.engine.is_key_pressed(K_LEFT)):
            self.object.vel += vector ([-power,0])*t
        if (self.engine.is_key_pressed(K_RIGHT)):
            self.object.vel += vector ([power,0])*t
    
        
gravity = vector ([0,900])
        
def gravity_func (obj, t):
    obj.vel += gravity * t
    